@@ -15,7 +15,9 @@
 #include <QApplication>
 #include <QThread>
 #include <QDebug>
-#include <QX11Info>
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
+#    include <QX11Info>
+#endif
 #include <QFile>
 #include <QProcess>
 #include <QDBusConnectionInterface>
@@ -42,6 +44,10 @@
 #    define DDE_LOCKSERVICE_SERVICE "org.deepin.dde.LockService1"
 #    define DDE_LOCKSERVICE_PATH "/org/deepin/dde/LockService1"
 #    define DDE_LOCKSERVICE_INTERFACE "org.deepin.dde.LockService1"
+
+#    define DESKTOP_FILEMONITOR_SERVICE "org.deepin.dde.desktop.filemonitor"
+#    define DESKTOP_FILEMONITOR_PATH "/org/deepin/dde/desktop/filemonitor"
+#    define DESKTOP_FILEMONITOR_INTERFACE "org.deepin.dde.desktop.filemonitor"
 #else
 #    define APP_MANAGER_SERVICE "com.deepin.SessionManager"
 #    define APP_MANAGER_PATH "/com/deepin/StartManager"
@@ -62,6 +68,10 @@
 #    define DDE_LOCKSERVICE_SERVICE "com.deepin.dde.LockService"
 #    define DDE_LOCKSERVICE_PATH "/com/deepin/dde/LockService"
 #    define DDE_LOCKSERVICE_INTERFACE "com.deepin.dde.LockService"
+
+#    define DESKTOP_FILEMONITOR_SERVICE "com.deepin.dde.desktop.filemonitor"
+#    define DESKTOP_FILEMONITOR_PATH "/com/deepin/dde/desktop/filemonitor"
+#    define DESKTOP_FILEMONITOR_INTERFACE "com.deepin.dde.desktop.filemonitor"
 #endif
 
 namespace dfmbase {
@@ -293,7 +303,7 @@ bool UniversalUtils::checkLaunchAppInterface()
 bool UniversalUtils::launchAppByDBus(const QString &desktopFile, const QStringList &filePaths)
 {
 #ifdef COMPILE_ON_V23
-    const auto &file = QFileInfo{desktopFile};
+    const auto &file = QFileInfo { desktopFile };
     constexpr auto kDesktopSuffix { u8"desktop" };
 
     if (file.suffix() != kDesktopSuffix) {
@@ -552,4 +562,23 @@ QString UniversalUtils::covertUrlToLocalPath(const QString &url)
         return QUrl(QUrl::fromPercentEncoding(url.toUtf8())).toLocalFile();
 }
 
+void UniversalUtils::boardCastPastData(const QUrl &sourcPath, const QUrl &targetPath, const QList<QUrl> &files)
+{
+    QDBusInterface fileMonitor(DESKTOP_FILEMONITOR_SERVICE,
+                               DESKTOP_FILEMONITOR_PATH,
+                               DESKTOP_FILEMONITOR_INTERFACE,
+                               QDBusConnection::sessionBus());
+
+    QList<QVariant> data;
+    data.append(sourcPath.toString());
+    data.append(targetPath.toString());
+    QStringList fileNames;
+    for (const auto &file : files) {
+        fileNames.append(file.fileName());
+    }
+
+    data.append(fileNames);
+    fileMonitor.asyncCallWithArgumentList(QStringLiteral("PrepareSendData"), data);
+}
+
 }
