@@ -78,53 +78,6 @@ void DoCopyFileWorker::doFileCopy(const DFileInfoPointer fromInfo, const DFileIn
     workData->completeFileCount++;
 }
 
-void DoCopyFileWorker::doMemcpyLocalBigFile(const DFileInfoPointer fromInfo, const DFileInfoPointer toInfo, char *dest, char *source, size_t size)
-{
-    size_t copySize = size;
-    char *destStart = dest;
-    char *sourceStart = source;
-    size_t everyCopySize = kMaxBufferLength;
-    while (copySize > 0 && !isStopped()) {
-        if (Q_UNLIKELY(!stateCheck())) {
-            break;
-        }
-        everyCopySize = copySize >= everyCopySize ? everyCopySize : copySize;
-
-        AbstractJobHandler::SupportAction action { AbstractJobHandler::SupportAction::kNoAction };
-
-        do {
-            action = AbstractJobHandler::SupportAction::kNoAction;
-            if (!memcpy(destStart, sourceStart, everyCopySize)) {
-                auto lastError = strerror(errno);
-                fmWarning() << "file memcpy error, url from: " << fromInfo->uri()
-                           << " url to: " << toInfo->uri()
-                           << " error code: " << errno << " error msg: " << lastError;
-
-                action = doHandleErrorAndWait(fromInfo->uri(), toInfo->uri(),
-                                              AbstractJobHandler::JobErrorType::kWriteError,
-                                              true, lastError);
-            }
-        } while (action == AbstractJobHandler::SupportAction::kRetryAction && !isStopped());
-
-        checkRetry();
-
-        if (!actionOperating(action, static_cast<qint64>(copySize), nullptr)) {
-            if (action == AbstractJobHandler::SupportAction::kSkipAction)
-                emit skipCopyLocalBigFile(fromInfo->uri());
-            return;
-        }
-
-        copySize -= everyCopySize;
-        destStart += everyCopySize;
-        sourceStart += everyCopySize;
-
-        if (memcpySkipUrl.isValid() && memcpySkipUrl == fromInfo->uri())
-            return;
-
-        workData->currentWriteSize += static_cast<int64_t>(everyCopySize);
-    }
-}
-
 bool DoCopyFileWorker::doDfmioFileCopy(const DFileInfoPointer fromInfo,
                                        const DFileInfoPointer toInfo, bool *skip)
 {
@@ -139,15 +92,15 @@ bool DoCopyFileWorker::doDfmioFileCopy(const DFileInfoPointer fromInfo,
         return false;
     // emit current task url
     auto fromUrl = fromInfo->uri();
-    auto toUrl =  toInfo->uri();
+    auto toUrl = toInfo->uri();
     emit currentTask(fromUrl, toUrl);
     // 创建doperator类
-    QSharedPointer<dfmio::DOperator> op{new dfmio::DOperator(fromUrl)};
+    QSharedPointer<dfmio::DOperator> op { new dfmio::DOperator(fromUrl) };
     fileOps.appendByLock(op);
-    ProgressData *data {new ProgressData()};
+    ProgressData *data { new ProgressData() };
     data->data = workData;
     data->copyFile = fromUrl;
-    bool ret{ false };
+    bool ret { false };
 
     DFile::CopyFlags flag = DFile::CopyFlag::kNoFollowSymlinks | DFile::CopyFlag::kOverwrite;
     if (!DeviceUtils::supportSetPermissionsDevice(toUrl))
@@ -159,8 +112,8 @@ bool DoCopyFileWorker::doDfmioFileCopy(const DFileInfoPointer fromInfo,
         if (!ret) {
             auto lastError = op->lastError().errorMsg();
             fmWarning() << "file copy error, url from: " << fromUrl
-                       << " url to: " << toUrl
-                       << " error code: " << op->lastError().code() << " error msg: " << lastError;
+                        << " url to: " << toUrl
+                        << " error code: " << op->lastError().code() << " error msg: " << lastError;
 
             action = doHandleErrorAndWait(fromUrl, toUrl,
                                           AbstractJobHandler::JobErrorType::kDfmIoError, false, lastError);
@@ -207,6 +160,49 @@ void DoCopyFileWorker::syncBlockFile(const DFileInfoPointer toInfo)
     }
 }
 
+/*!
+ * \brief DoCopyFileWorker::openFile
+ * \param fromInfo
+ * \param toInfo
+ * \param flags
+ * \param skip
+ * \param isSource
+ * \return
+ */
+int DoCopyFileWorker::openFileBySys(const DFileInfoPointer &fromInfo, const DFileInfoPointer &toInfo,
+                                    const int flags, bool *skip, const bool isSource)
+{
+    int fd = -1;
+    AbstractJobHandler::SupportAction action = AbstractJobHandler::SupportAction::kNoAction;
+    auto openUrl = isSource ? fromInfo->uri() : toInfo->uri();
+    do {
+        action = AbstractJobHandler::SupportAction::kNoAction;
+        if (flags & O_CREAT) {
+            fd = open(openUrl.path().toStdString().c_str(), flags, 0666);
+        } else {
+            fd = open(openUrl.path().toStdString().c_str(), flags);
+        }
+        if (fd < 0) {
+            auto lastError = strerror(errno);
+            fmWarning() << "file open error, url from: " << fromInfo->uri()
+                       << " url to: " << toInfo->uri() << " open flag: " << flags
+                       << " open url : " << openUrl << " error msg: " << lastError;
+            action = doHandleErrorAndWait(fromInfo->uri(), toInfo->uri(),
+                                          AbstractJobHandler::JobErrorType::kOpenError,
+                                          !isSource, lastError);
+        }
+    } while (action == AbstractJobHandler::SupportAction::kRetryAction && !isStopped());
+    checkRetry();
+    auto fileSize = fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
+    if (!actionOperating(action, fileSize <= 0 ? FileUtils::getMemoryPageSize() : fileSize, skip)) {
+        close(fd);
+        return -1;
+    }
+    if (isSource && fileSize > 100 * 1024 * 1024)
+        readahead(fd, 0, static_cast<size_t>(fileSize));
+    return fd;
+}
+
 // copy thread using
 DoCopyFileWorker::NextDo DoCopyFileWorker::doCopyFilePractically(const DFileInfoPointer fromInfo, const DFileInfoPointer toInfo, bool *skip)
 {
@@ -230,7 +226,7 @@ DoCopyFileWorker::NextDo DoCopyFileWorker::doCopyFilePractically(const DFileInfo
         setTargetPermissions(fromInfo->uri(), toInfo->uri());
         workData->zeroOrlinkOrDirWriteSize += FileUtils::getMemoryPageSize();
         FileUtils::notifyFileChangeManual(DFMBASE_NAMESPACE::Global::FileNotifyType::kFileAdded, toInfo->uri());
-        if (workData->exBlockSyncEveryWrite)
+        if (workData->exBlockSyncEveryWrite || DeviceUtils::isSamba(toInfo->uri()))
             syncBlockFile(toInfo);
         return NextDo::kDoCopyNext;
     }
@@ -239,7 +235,8 @@ DoCopyFileWorker::NextDo DoCopyFileWorker::doCopyFilePractically(const DFileInfo
         return NextDo::kDoCopyErrorAddCancel;
     // 循环读取和写入文件，拷贝
     int toFd = -1;
-    if (workData->exBlockSyncEveryWrite)
+    auto toIsSmb = DeviceUtils::isSamba(toInfo->uri());
+    if (workData->exBlockSyncEveryWrite || toIsSmb)
         toFd = open(toInfo->uri().path().toUtf8().toStdString().data(), O_RDONLY);
     qint64 blockSize = fromSize > kMaxBufferLength ? kMaxBufferLength : fromSize;
     char *data = new char[static_cast<uint>(blockSize + 1)];
@@ -269,7 +266,7 @@ DoCopyFileWorker::NextDo DoCopyFileWorker::doCopyFilePractically(const DFileInfo
         }
 
         // 执行同步策略
-        if (workData->exBlockSyncEveryWrite && toFd > 0)
+        if ((workData->exBlockSyncEveryWrite || toIsSmb) && toFd > 0)
             syncfs(toFd);
 
     } while (fromDevice->pos() != fromSize);
@@ -278,7 +275,7 @@ DoCopyFileWorker::NextDo DoCopyFileWorker::doCopyFilePractically(const DFileInfo
     data = nullptr;
 
     // 执行同步策略
-    if (workData->exBlockSyncEveryWrite && toFd > 0)
+    if ((workData->exBlockSyncEveryWrite  || toIsSmb) && toFd > 0)
         syncfs(toFd);
 
     if (toFd > 0)
@@ -300,6 +297,90 @@ DoCopyFileWorker::NextDo DoCopyFileWorker::doCopyFilePractically(const DFileInfo
     return NextDo::kDoCopyNext;
 }
 
+/*!
+ * \brief DoCopyFileWorker::doCopyFileByRange
+ * \param fromInfo
+ * \param toInfo
+ * \param skip
+ * \return
+ */
+DoCopyFileWorker::NextDo DoCopyFileWorker::doCopyFileByRange(const DFileInfoPointer fromInfo, const DFileInfoPointer toInfo, bool *skip)
+{
+    if (isStopped())
+        return NextDo::kDoCopyErrorAddCancel;
+    // emit current task url
+    emit currentTask(fromInfo->uri(), toInfo->uri());
+    // open source file
+    int sourcFd = openFileBySys(fromInfo, toInfo, O_RDONLY, skip);
+    if (sourcFd < 0)
+        return NextDo::kDoCopyErrorAddCancel;
+    FinallyUtil releaseSc([&] {
+        close(sourcFd);
+    });
+    int targetFd = openFileBySys(fromInfo, toInfo, O_CREAT | O_WRONLY | O_TRUNC, skip, false);
+    if (targetFd < 0) {
+        return NextDo::kDoCopyErrorAddCancel;
+    }
+    FinallyUtil releaseTg([&] {
+        close(targetFd);
+    });
+    // 源文件大小如果为0
+    auto fromSize = fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
+    if (fromSize <= 0) {
+        // 对文件加权
+        setTargetPermissions(fromInfo->uri(), toInfo->uri());
+        workData->zeroOrlinkOrDirWriteSize += FileUtils::getMemoryPageSize();
+        FileUtils::notifyFileChangeManual(DFMBASE_NAMESPACE::Global::FileNotifyType::kFileAdded, toInfo->uri());
+        if (workData->exBlockSyncEveryWrite || DeviceUtils::isSamba(toInfo->uri()))
+            syncfs(targetFd);
+        return NextDo::kDoCopyNext;
+    }
+    // 循环读取和写入文件，拷贝
+    auto toIsSmb = DeviceUtils::isSamba(toInfo->uri());
+    size_t blockSize = static_cast<size_t>(fromSize > kMaxBufferLength ? kMaxBufferLength : fromSize);
+    off_t offset_in = 0;
+    off_t offset_out = 0;
+    ssize_t result = -1;
+    AbstractJobHandler::SupportAction action { AbstractJobHandler::SupportAction::kNoAction };
+    do {
+        if (Q_UNLIKELY(!stateCheck()))
+            return NextDo::kDoCopyErrorAddCancel;
+        do {
+            if (Q_UNLIKELY(!stateCheck()))
+                return NextDo::kDoCopyErrorAddCancel;
+            result = copy_file_range(sourcFd, &offset_in, targetFd, &offset_out, blockSize, 0);
+            if (result < 0) {
+                auto lastError = strerror(errno);
+                fmWarning() << "copy file range error, url from: " << fromInfo->uri()
+                           << " url to: " << toInfo->uri() << " error msg: " << lastError;
+                action = doHandleErrorAndWait(fromInfo->uri(), toInfo->uri(),
+                                              AbstractJobHandler::JobErrorType::kWriteError,
+                                              false, lastError);
+                offset_in = qMin(offset_in, offset_out);
+                offset_out = offset_in;
+            } else {
+                workData->currentWriteSize += result;
+            }
+        } while (action == AbstractJobHandler::SupportAction::kRetryAction && !isStopped());
+        checkRetry();
+        if (!actionOperating(action, fromSize - offset_out, skip))
+            return  NextDo::kDoCopyErrorAddCancel;
+        // 执行同步策略
+        if (workData->exBlockSyncEveryWrite || toIsSmb)
+            syncfs(targetFd);
+    } while (offset_out != fromSize);
+    // 执行同步策略
+    if (workData->exBlockSyncEveryWrite  || toIsSmb)
+        syncfs(targetFd);
+    // 对文件加权
+    setTargetPermissions(fromInfo->uri(), toInfo->uri());
+    if (!stateCheck())
+        return NextDo::kDoCopyErrorAddCancel;
+    if (skip && *skip)
+        FileUtils::notifyFileChangeManual(DFMBASE_NAMESPACE::Global::FileNotifyType::kFileAdded, toInfo->uri());
+    return NextDo::kDoCopyNext;
+}
+
 bool DoCopyFileWorker::stateCheck()
 {
     if (state == kPasued)
@@ -349,16 +430,14 @@ AbstractJobHandler::SupportAction DoCopyFileWorker::doHandleErrorAndWait(const Q
 
     bool isSame = false, checkAgain = true;
     if (urlFrom.scheme() == Global::Scheme::kFile
-            &&  urlFrom.scheme() == Global::Scheme::kFile) {
+        && urlFrom.scheme() == Global::Scheme::kFile) {
         auto tmF = FileUtils::bindUrlTransform(urlFrom);
         auto tmT = FileUtils::bindUrlTransform(urlTo);
         isSame = UniversalUtils::urlEquals(tmF, tmT);
         checkAgain = false;
     }
 
-    if (isSame || (checkAgain &&
-                   FileUtils::isSameFile(urlFrom, urlTo,
-                                         Global::CreateFileInfoType::kCreateFileInfoSync))) {
+    if (isSame || (checkAgain && FileUtils::isSameFile(urlFrom, urlTo, Global::CreateFileInfoType::kCreateFileInfoSync))) {
         currentAction = AbstractJobHandler::SupportAction::kCoexistAction;
         return currentAction;
     }
@@ -491,8 +570,8 @@ bool DoCopyFileWorker::openFile(const DFileInfoPointer &fromInfo, const DFileInf
         if (!file->open(flags)) {
             auto lastError = file->lastError();
             fmWarning() << "file open error, url from: " << fromInfo->uri()
-                       << " url to: " << toInfo->uri() << " open flag: " << flags
-                       << " error code: " << lastError.code() << " error msg: " << lastError.errorMsg();
+                        << " url to: " << toInfo->uri() << " open flag: " << flags
+                        << " error code: " << lastError.code() << " error msg: " << lastError.errorMsg();
 
             action = doHandleErrorAndWait(fromInfo->uri(), toInfo->uri(),
                                           AbstractJobHandler::JobErrorType::kOpenError,
@@ -584,14 +663,13 @@ DoCopyFileWorker::NextDo DoCopyFileWorker::doReadFile(const DFileInfoPointer &fr
                     if (!NetworkUtils::instance()->checkFtpOrSmbBusy(fromInfo->uri())) {
                         break;
                     }
-                    actionForCheck
-                            = doHandleErrorAndWait(
-                                fromInfo->uri(),
-                                toInfo->uri(),
-                                AbstractJobHandler::JobErrorType::kCanNotAccessFile,
-                                true,
-                                "Can't access file!");
-                } while(actionForCheck == AbstractJobHandler::SupportAction::kRetryAction && !isStopped());
+                    actionForCheck = doHandleErrorAndWait(
+                            fromInfo->uri(),
+                            toInfo->uri(),
+                            AbstractJobHandler::JobErrorType::kCanNotAccessFile,
+                            true,
+                            "Can't access file!");
+                } while (actionForCheck == AbstractJobHandler::SupportAction::kRetryAction && !isStopped());
                 if (actionForCheck != AbstractJobHandler::SupportAction::kNoAction) {
                     if (skip)
                         *skip = actionForCheck == AbstractJobHandler::SupportAction::kSkipAction;
@@ -662,7 +740,7 @@ DoCopyFileWorker::NextDo DoCopyFileWorker::doWriteFile(const DFileInfoPointer &f
                                               toDevice->lastError().errorMsg());
         if (actionForWrite == AbstractJobHandler::SupportAction::kRetryAction && !isStopped()) {
             qint64 curWrite = 0;
-            auto nextDo = doWriteFileErrorRetry(fromInfo,toInfo, toDevice, fromDevice, readSize, skip, currentPos, surplusSize, curWrite);
+            auto nextDo = doWriteFileErrorRetry(fromInfo, toInfo, toDevice, fromDevice, readSize, skip, currentPos, surplusSize, curWrite);
             checkRetry();
             return nextDo;
         }
@@ -670,33 +748,24 @@ DoCopyFileWorker::NextDo DoCopyFileWorker::doWriteFile(const DFileInfoPointer &f
 
     checkRetry();
 
-    if (!actionOperating(actionForWrite, fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong()
-                         - (currentPos + readSize - surplusSize), skip))
-        return  NextDo::kDoCopyErrorAddCancel;
+    if (!actionOperating(actionForWrite, fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() - (currentPos + readSize - surplusSize), skip))
+        return NextDo::kDoCopyErrorAddCancel;
 
     if (workData->needSyncEveryRW && sizeWrite > 0) {
-        if (workData->isFsTypeVfat) {
-            // FAT and VFAT file systems ignore the "sync" option
-            toDevice->close();
-            if (!openFile(fromInfo, toInfo, toDevice, DFMIO::DFile::OpenFlag::kWriteOnly | DFMIO::DFile::OpenFlag::kAppend, skip)) {
-                return  NextDo::kDoCopyErrorAddCancel;
-            }
-        } else {
-            toDevice->flush();
-        }
+        toDevice->flush();
     }
 
-    return  NextDo::kDoCopyCurrentFile;
+    return NextDo::kDoCopyCurrentFile;
 }
 
 DoCopyFileWorker::NextDo DoCopyFileWorker::doWriteFileErrorRetry(const DFileInfoPointer &fromInfo,
-                                                            const DFileInfoPointer &toInfo,
-                                                            const QSharedPointer<DFile> &toDevice,
-                                                            const QSharedPointer<DFile> &fromDevice,
-                                                            const qint64 readSize,
-                                                            bool *skip,
-                                                            const qint64 currentPos,
-                                                            const qint64 &surplusSize, qint64 &curWrite)
+                                                                 const DFileInfoPointer &toInfo,
+                                                                 const QSharedPointer<DFile> &toDevice,
+                                                                 const QSharedPointer<DFile> &fromDevice,
+                                                                 const qint64 readSize,
+                                                                 bool *skip,
+                                                                 const qint64 currentPos,
+                                                                 const qint64 &surplusSize, qint64 &curWrite)
 {
     Q_UNUSED(toDevice);
     Q_UNUSED(fromDevice);
@@ -708,17 +777,15 @@ DoCopyFileWorker::NextDo DoCopyFileWorker::doWriteFileErrorRetry(const DFileInfo
         if (!NetworkUtils::instance()->checkFtpOrSmbBusy(toInfo->uri())) {
             break;
         }
-        actionForWrite
-                = doHandleErrorAndWait(
-                    fromInfo->uri(),
-                    toInfo->uri(),
-                    AbstractJobHandler::JobErrorType::kCanNotAccessFile,
-                    true,
-                    "Can't access file!");
-    } while(actionForWrite == AbstractJobHandler::SupportAction::kRetryAction && !isStopped());
+        actionForWrite = doHandleErrorAndWait(
+                fromInfo->uri(),
+                toInfo->uri(),
+                AbstractJobHandler::JobErrorType::kCanNotAccessFile,
+                true,
+                "Can't access file!");
+    } while (actionForWrite == AbstractJobHandler::SupportAction::kRetryAction && !isStopped());
     if (actionForWrite != AbstractJobHandler::SupportAction::kNoAction) {
-        actionOperating(actionForWrite, fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong()
-                        - (currentPos + readSize - surplusSize), skip);
+        actionOperating(actionForWrite, fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() - (currentPos + readSize - surplusSize), skip);
         return NextDo::kDoCopyErrorAddCancel;
     }
 
@@ -733,7 +800,7 @@ bool DoCopyFileWorker::verifyFileIntegrity(const qint64 &blockSize, const ulong
     if (!workData->jobFlags.testFlag(AbstractJobHandler::JobFlag::kCopyIntegrityChecking))
         return true;
     char *data = new char[static_cast<uint>(blockSize + 1)];
-    QTime t;
+    QElapsedTimer t;
     ulong targetCheckSum = adler32(0L, nullptr, 0);
     Q_FOREVER {
         qint64 size = toDevice->read(data, blockSize);
