@@ -81,7 +81,8 @@ void FileOperateBaseWorker::emitSpeedUpdatedNotify(const qint64 &writSize)
     }
 
     qint64 speed = currentState == AbstractJobHandler::JobState::kRunningState
-            ? writSize * 1000 / (elTime) : 0;
+            ? writSize * 1000 / (elTime)
+            : 0;
     info->insert(AbstractJobHandler::NotifyInfoKey::kJobtypeKey, QVariant::fromValue(jobType));
     info->insert(AbstractJobHandler::NotifyInfoKey::kJobStateKey, QVariant::fromValue(currentState));
     info->insert(AbstractJobHandler::NotifyInfoKey::kSpeedKey, QVariant::fromValue(speed));
@@ -106,7 +107,7 @@ void FileOperateBaseWorker::setTargetPermissions(const QUrl &fromUrl, const QUrl
                                   fromInfo->timeOf(TimeInfoType::kLastModified).value<QDateTime>());
     QFileDevice::Permissions permissions = fromInfo->permissions();
     QString path = fromInfo->urlOf(UrlInfoType::kUrl).path();
-    //权限为0000时，源文件已经被删除，无需修改新建的文件的权限为0000
+    // 权限为0000时，源文件已经被删除，无需修改新建的文件的权限为0000
     if (permissions != 0000 && !FileUtils::isMtpFile(toInfo->urlOf(UrlInfoType::kUrl)))
         localFileHandler->setPermissions(toInfo->urlOf(UrlInfoType::kUrl), permissions);
 }
@@ -348,7 +349,8 @@ bool FileOperateBaseWorker::copyAndDeleteFile(const DFileInfoPointer &fromInfo,
         ok = createSystemLink(fromInfo, toInfo, workData->jobFlags.testFlag(AbstractJobHandler::JobFlag::kCopyFollowSymlink), true, skip);
         if (ok) {
             workData->zeroOrlinkOrDirWriteSize += FileUtils::getMemoryPageSize();
-            cutAndDeleteFiles.append(fromInfo);
+            if (!skip || !*skip)
+                cutAndDeleteFiles.append(fromInfo);
         }
     } else if (fromInfo->attribute(DFileInfo::AttributeID::kStandardIsDir).toBool()) {
         ok = checkAndCopyDir(fromInfo, toInfo, skip);
@@ -368,7 +370,7 @@ bool FileOperateBaseWorker::copyAndDeleteFile(const DFileInfoPointer &fromInfo,
         if (fromSize > bigFileSize || !supportDfmioCopy || workData->exBlockSyncEveryWrite) {
             do {
                 nextDo = copyOtherFileWorker->doCopyFilePractically(fromInfo, toInfo, skip);
-            } while( nextDo == DoCopyFileWorker::NextDo::kDoCopyReDoCurrentFile && !isStopped());
+            } while (nextDo == DoCopyFileWorker::NextDo::kDoCopyReDoCurrentFile && !isStopped());
             ok = nextDo != DoCopyFileWorker::NextDo::kDoCopyErrorAddCancel;
         } else {
             ok = copyOtherFileWorker->doDfmioFileCopy(fromInfo, toInfo, skip);
@@ -444,7 +446,8 @@ DFileInfoPointer FileOperateBaseWorker::doCheckFile(const DFileInfoPointer &from
         setSkipValue(skip, action);
         if (skip && *skip)
             workData->skipWriteSize += fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() <= 0
-                    ? workData->dirSize : fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
+                    ? workData->dirSize
+                    : fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
 
         return nullptr;
     }
@@ -516,7 +519,14 @@ bool FileOperateBaseWorker::createSystemLink(const DFileInfoPointer &fromInfo, c
 
     do {
         actionForlink = AbstractJobHandler::SupportAction::kNoAction;
-        if (localFileHandler->createSystemLink(newFromInfo->uri(), toInfo->uri())) {
+
+        DFileInfo::AttributeID attributeId = newFromInfo->attribute(DFileInfo::AttributeID::kStandardIsSymlink).toBool()
+                ? DFileInfo::AttributeID::kStandardSymlinkTarget
+                : DFileInfo::AttributeID::kStandardFilePath;
+
+        auto target = QUrl::fromLocalFile(newFromInfo->attribute(attributeId).toString());
+
+        if (localFileHandler->createSystemLink(target, toInfo->uri())) {
             return true;
         }
         actionForlink = doHandleErrorAndWait(fromInfo->uri(), toInfo->uri(),
@@ -542,7 +552,7 @@ DFileInfoPointer FileOperateBaseWorker::doCheckNewFile(const DFileInfoPointer &f
                                                        bool isCountSize)
 {
     auto newTargetUrl = createNewTargetUrl(toInfo, fileNewName);
-    DFileInfoPointer newTargetInfo { new DFileInfo(newTargetUrl)};
+    DFileInfoPointer newTargetInfo { new DFileInfo(newTargetUrl) };
     newTargetInfo->initQuerier();
     if (!newTargetInfo->exists())
         return newTargetInfo;
@@ -555,9 +565,10 @@ DFileInfoPointer FileOperateBaseWorker::doCheckNewFile(const DFileInfoPointer &f
             setSkipValue(skip, action);
             if (skip && *skip)
                 workData->skipWriteSize += isCountSize
-                        && (fromInfo->attribute(DFileInfo::AttributeID::kStandardIsSymlink).toBool()
-                            || fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() <= 0)
-                        ? workData->dirSize : fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
+                                && (fromInfo->attribute(DFileInfo::AttributeID::kStandardIsSymlink).toBool()
+                                    || fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() <= 0)
+                        ? workData->dirSize
+                        : fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
             return nullptr;
         }
 
@@ -565,9 +576,10 @@ DFileInfoPointer FileOperateBaseWorker::doCheckNewFile(const DFileInfoPointer &f
             setSkipValue(skip, action);
             if (skip && *skip)
                 workData->skipWriteSize += isCountSize
-                        && (fromInfo->attribute(DFileInfo::AttributeID::kStandardIsSymlink).toBool()
-                            || fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() <= 0)
-                        ? workData->dirSize : fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
+                                && (fromInfo->attribute(DFileInfo::AttributeID::kStandardIsSymlink).toBool()
+                                    || fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() <= 0)
+                        ? workData->dirSize
+                        : fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
             return nullptr;
         }
     };
@@ -589,13 +601,15 @@ DFileInfoPointer FileOperateBaseWorker::doCheckNewFile(const DFileInfoPointer &f
         const QVariant &var = doActionMerge(fromInfo, newTargetInfo, isCountSize);
         if (var.isValid())
             return var.toBool() ? newTargetInfo : nullptr;
+        newTargetInfo->initQuerier();
+        if (jobType == AbstractJobHandler::JobType::kCutType && newTargetInfo)
+            isCutMerge = true;
         break;
     }
     case AbstractJobHandler::SupportAction::kSkipAction: {
-        workData->skipWriteSize += (isCountSize &&
-                                    (fromInfo->attribute(DFileInfo::AttributeID::kStandardIsSymlink).toBool()
-                                     || fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() <= 0))
-                ? workData->dirSize : fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
+        workData->skipWriteSize += (isCountSize && (fromInfo->attribute(DFileInfo::AttributeID::kStandardIsSymlink).toBool() || fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() <= 0))
+                ? workData->dirSize
+                : fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
         setSkipValue(skip, action);
         return nullptr;
     }
@@ -636,9 +650,9 @@ bool FileOperateBaseWorker::checkAndCopyFile(const DFileInfoPointer fromInfo, co
         while (bigFileCopy && !isStopped()) {
             QThread::msleep(10);
         }
-        if (fromSize > bigFileSize) {
+        if (fromSize > bigFileSize && FileUtils::isSameDevice(fromInfo->uri(), targetUrl)) {
             bigFileCopy = true;
-            auto result = doCopyLocalBigFile(fromInfo, toInfo, skip);
+            auto result = doCopyLocalByRange(fromInfo, toInfo, skip);
             bigFileCopy = false;
             return result;
         }
@@ -663,10 +677,10 @@ bool FileOperateBaseWorker::checkAndCopyDir(const DFileInfoPointer &fromInfo, co
                 break;
             // 特殊处理
             auto errstr = localFileHandler->errorString();
-            auto fileUrl= toInfo->uri();
+            auto fileUrl = toInfo->uri();
             if (localFileHandler->errorCode() == DFMIOErrorCode::DFM_IO_ERROR_FAILED
-                    && fileUrl.path().toLocal8Bit().size() > 255
-                    && FileUtils::isMtpFile(fileUrl))
+                && fileUrl.path().toLocal8Bit().size() > 255
+                && FileUtils::isMtpFile(fileUrl))
                 errstr = tr("The file name or the path is too long!");
 
             action = doHandleErrorAndWait(fromInfo->uri(), toInfo->uri(),
@@ -705,6 +719,7 @@ bool FileOperateBaseWorker::checkAndCopyDir(const DFileInfoPointer &fromInfo, co
     }
 
     bool self = true;
+    iterator->setProperty("QueryAttributes", "standard::name");
     while (iterator->hasNext()) {
         if (!stateCheck()) {
             return false;
@@ -712,6 +727,7 @@ bool FileOperateBaseWorker::checkAndCopyDir(const DFileInfoPointer &fromInfo, co
 
         const QUrl &url = iterator->next();
         DFileInfoPointer info(new DFileInfo(url));
+        info->initQuerier();
         bool ok = doCopyFile(info, toInfo, skip);
         if (!ok && (!skip || !*skip)) {
             return false;
@@ -724,7 +740,7 @@ bool FileOperateBaseWorker::checkAndCopyDir(const DFileInfoPointer &fromInfo, co
             }
 
             if (info->attribute(DFileInfo::AttributeID::kStandardIsSymlink).toBool()
-                    || info->attribute(DFileInfo::AttributeID::kStandardIsFile).toBool()) {
+                || info->attribute(DFileInfo::AttributeID::kStandardIsFile).toBool()) {
                 cutAndDeleteFiles.append(info);
             } else if (self && !cutAndDeleteFiles.contains(info)) {
                 self = false;
@@ -871,174 +887,29 @@ bool FileOperateBaseWorker::doCopyLocalFile(const DFileInfoPointer fromInfo, con
     if (!stateCheck())
         return false;
 
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
     QtConcurrent::run(threadPool.data(), threadCopyWorker[threadCopyFileCount % threadCount].data(),
                       static_cast<void (DoCopyFileWorker::*)(const DFileInfoPointer, const DFileInfoPointer)>(&DoCopyFileWorker::doFileCopy),
                       fromInfo, toInfo);
+#else
+    QtConcurrent::run(threadPool.data(), [this, fromInfo, toInfo]() {
+        threadCopyWorker[threadCopyFileCount % threadCount]->doFileCopy(fromInfo, toInfo);
+    });
+#endif
 
     threadCopyFileCount++;
     return true;
 }
 
-bool FileOperateBaseWorker::doCopyLocalBigFile(const DFileInfoPointer fromInfo, const DFileInfoPointer toInfo, bool *skip)
+bool FileOperateBaseWorker::doCopyLocalByRange(const DFileInfoPointer fromInfo, const DFileInfoPointer toInfo, bool *skip)
 {
     waitThreadPoolOver();
-    // open file
-    auto fromFd = doOpenFile(fromInfo, toInfo, false, O_RDONLY, skip);
-    if (fromFd < 0)
-        return false;
-    auto toFd = doOpenFile(fromInfo, toInfo, true, O_CREAT | O_RDWR, skip);
-    if (toFd < 0) {
-        close(fromFd);
-        return false;
-    }
-    // resize target file
-    if (!doCopyLocalBigFileResize(fromInfo, toInfo, toFd, skip)) {
-        close(fromFd);
-        close(toFd);
-        return false;
-    }
-    // mmap file
-    auto fromPoint = doCopyLocalBigFileMap(fromInfo, toInfo, fromFd, PROT_READ, skip);
-    if (!fromPoint) {
-        close(fromFd);
-        close(toFd);
-        return false;
-    }
-    auto toPoint = doCopyLocalBigFileMap(fromInfo, toInfo, toFd, PROT_WRITE, skip);
-    if (!toPoint) {
-        munmap(fromPoint, static_cast<size_t>(fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong()));
-        close(fromFd);
-        close(toFd);
-        return false;
-    }
-    // memcpy file in other thread
-    memcpyLocalBigFile(fromInfo, toInfo, fromPoint, toPoint);
-    // wait copy
-    waitThreadPoolOver();
-    // clear
-    doCopyLocalBigFileClear(static_cast<size_t>(fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong()), fromFd, toFd, fromPoint, toPoint);
-    // set permissions
-    setTargetPermissions(fromInfo->uri(), toInfo->uri());
-    return true;
-}
+    const QString &targetUrl = toInfo->uri().toString();
 
-bool FileOperateBaseWorker::doCopyLocalBigFileResize(const DFileInfoPointer fromInfo, const DFileInfoPointer toInfo, int toFd, bool *skip)
-{
-    AbstractJobHandler::SupportAction action { AbstractJobHandler::SupportAction::kNoAction };
-    __off_t length = fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
-    do {
-        action = AbstractJobHandler::SupportAction::kNoAction;
-        if (-1 == ftruncate(toFd, length)) {
-            auto lastError = strerror(errno);
-            fmWarning() << "file resize error, url from: " << fromInfo->uri()
-                        << " url to: " << toInfo->uri() << " open flag: " << O_RDONLY
-                        << " error code: " << errno << " error msg: " << lastError;
-
-            action = doHandleErrorAndWait(fromInfo->uri(), toInfo->uri(),
-                                          AbstractJobHandler::JobErrorType::kResizeError, true, lastError);
-        }
-    } while (action == AbstractJobHandler::SupportAction::kRetryAction && !isStopped());
-
-    checkRetry();
-
-    if (!actionOperating(action, length <= 0 ? FileUtils::getMemoryPageSize() : length, skip))
-        return false;
-
-    return true;
-}
-
-char *FileOperateBaseWorker::doCopyLocalBigFileMap(const DFileInfoPointer fromInfo, const DFileInfoPointer toInfo, int fd, const int per, bool *skip)
-{
-    AbstractJobHandler::SupportAction action { AbstractJobHandler::SupportAction::kNoAction };
-    void *point = nullptr;
-    auto fromSize = fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
-    do {
-        action = AbstractJobHandler::SupportAction::kNoAction;
-        point = mmap(nullptr, static_cast<size_t>(fromSize),
-                     per, MAP_SHARED, fd, 0);
-        if (!point || point == MAP_FAILED) {
-            auto lastError = strerror(errno);
-            fmWarning() << "file mmap error, url from: " << fromInfo->uri()
-                        << " url to: " << fromInfo->uri()
-                        << " error code: " << errno << " error msg: " << lastError;
-
-            action = doHandleErrorAndWait(fromInfo->uri(), toInfo->uri(),
-                                          AbstractJobHandler::JobErrorType::kOpenError, fd == PROT_WRITE, lastError);
-        }
-    } while (action == AbstractJobHandler::SupportAction::kRetryAction && !isStopped());
-
-    checkRetry();
-
-    if (!actionOperating(action, fromSize <= 0 ? FileUtils::getMemoryPageSize() : fromSize, skip))
-        return nullptr;
-
-    return static_cast<char *>(point);
-}
-
-void FileOperateBaseWorker::memcpyLocalBigFile(const DFileInfoPointer fromInfo, const DFileInfoPointer toInfo, char *fromPoint, char *toPoint)
-{
-    auto fromSize = fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
-    auto offset = fromSize / threadCount;
-    char *fromPointStart = fromPoint;
-    char *toPointStart = toPoint;
-    for (int i = 0; i < threadCount; i++) {
-        offset = (i == (threadCount - 1) ? fromSize - (threadCount - 1) * offset : offset);
-
-        char *tempfFromPointStart = fromPointStart;
-        char *tempfToPointStart = toPointStart;
-        size_t tempOffet = static_cast<size_t>(offset);
-        QtConcurrent::run(threadPool.data(), threadCopyWorker[i].data(),
-                          static_cast<void (DoCopyFileWorker::*)(const DFileInfoPointer fromInfo,
-                                                                 const DFileInfoPointer toInfo,
-                                                                 char *dest, char *source, size_t size)>(&DoCopyFileWorker::doMemcpyLocalBigFile),
-                          fromInfo, toInfo, tempfToPointStart, tempfFromPointStart, tempOffet);
-
-        fromPointStart += offset;
-        toPointStart += offset;
-    }
-}
-
-void FileOperateBaseWorker::doCopyLocalBigFileClear(const size_t size,
-                                                    const int fromFd, const int toFd, char *fromPoint, char *toPoint)
-{
-    munmap(fromPoint, size);
-    munmap(toPoint, size);
-    close(fromFd);
-    close(toFd);
-}
-
-int FileOperateBaseWorker::doOpenFile(const DFileInfoPointer fromInfo, const DFileInfoPointer toInfo,
-                                      const bool isTo, const int openFlag, bool *skip)
-{
-    AbstractJobHandler::SupportAction action { AbstractJobHandler::SupportAction::kNoAction };
-    emitCurrentTaskNotify(fromInfo->uri(), toInfo->uri());
-    int fd = -1;
-    do {
-        QUrl url = isTo ? toInfo->uri() : fromInfo->uri();
-        std::string path = url.path().toStdString();
-        fd = open(path.c_str(), openFlag, 0666);
-        action = AbstractJobHandler::SupportAction::kNoAction;
-        if (fd < 0) {
-            auto lastError = strerror(errno);
-            fmWarning() << "file open error, url from: " << fromInfo->uri()
-                        << " url to: " << fromInfo->uri() << " open flag: " << openFlag
-                        << " error code: " << errno << " error msg: " << lastError;
-
-            action = doHandleErrorAndWait(fromInfo->uri(), toInfo->uri(),
-                                          AbstractJobHandler::JobErrorType::kOpenError, isTo, lastError);
-        }
-    } while (action == AbstractJobHandler::SupportAction::kRetryAction && !isStopped());
-
-    checkRetry();
-
-    auto fromSize = fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
-    if (!actionOperating(action, fromSize <= 0 ? FileUtils::getMemoryPageSize() : fromSize, skip)) {
-        if (fd >= 0)
-            close(fd);
-        return -1;
-    }
-
-    return fd;
+    FileUtils::cacheCopyingFileUrl(targetUrl);
+    DoCopyFileWorker::NextDo nextDo = threadCopyWorker[0]->doCopyFileByRange(fromInfo, toInfo, skip);
+    FileUtils::removeCopyingFileUrl(targetUrl);
+    return nextDo == DoCopyFileWorker::NextDo::kDoCopyNext;
 }
 
 bool FileOperateBaseWorker::doCopyOtherFile(const DFileInfoPointer fromInfo, const DFileInfoPointer toInfo, bool *skip)
@@ -1053,7 +924,7 @@ bool FileOperateBaseWorker::doCopyOtherFile(const DFileInfoPointer fromInfo, con
     if (fromSize > bigFileSize || !supportDfmioCopy || workData->exBlockSyncEveryWrite) {
         do {
             nextDo = copyOtherFileWorker->doCopyFilePractically(fromInfo, toInfo, skip);
-        } while( nextDo == DoCopyFileWorker::NextDo::kDoCopyReDoCurrentFile && !isStopped());
+        } while (nextDo == DoCopyFileWorker::NextDo::kDoCopyReDoCurrentFile && !isStopped());
         ok = nextDo != DoCopyFileWorker::NextDo::kDoCopyErrorAddCancel;
     } else {
         ok = copyOtherFileWorker->doDfmioFileCopy(fromInfo, toInfo, skip);
@@ -1113,10 +984,9 @@ QVariant FileOperateBaseWorker::checkLinkAndSameUrl(const DFileInfoPointer &from
 
     const QUrl &newTargetUrl = newTargetInfo->uri();
     if (newTargetUrl == fromInfo->uri()) {
-        workData->skipWriteSize += (isCountSize &&
-                                    (fromInfo->attribute(DFileInfo::AttributeID::kStandardIsSymlink).toBool()
-                                     || fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() <= 0))
-                ? workData->dirSize : fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
+        workData->skipWriteSize += (isCountSize && (fromInfo->attribute(DFileInfo::AttributeID::kStandardIsSymlink).toBool() || fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() <= 0))
+                ? workData->dirSize
+                : fromInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong();
         return true;
     }
 
@@ -1172,9 +1042,8 @@ bool FileOperateBaseWorker::doCopyFile(const DFileInfoPointer &fromInfo, const D
         result = createSystemLink(fromInfo, newTargetInfo, workData->jobFlags.testFlag(AbstractJobHandler::JobFlag::kCopyFollowSymlink), true, skip);
         if (result)
             workData->zeroOrlinkOrDirWriteSize +=
-                    (newTargetInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() > 0 ?
-                         newTargetInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong()
-                       : FileUtils::getMemoryPageSize());
+                    (newTargetInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong() > 0 ? newTargetInfo->attribute(DFileInfo::AttributeID::kStandardSize).toLongLong()
+                                                                                                      : FileUtils::getMemoryPageSize());
     } else if (fromInfo->attribute(DFileInfo::AttributeID::kStandardIsDir).toBool()) {
         result = checkAndCopyDir(fromInfo, newTargetInfo, skip);
         if (result || skip)
@@ -1183,6 +1052,10 @@ bool FileOperateBaseWorker::doCopyFile(const DFileInfoPointer &fromInfo, const D
         result = checkAndCopyFile(fromInfo, newTargetInfo, skip);
     }
 
+    if (result)
+        dpfSignalDispatcher->publish("dfmplugin_fileoperations", "signal_File_Add",
+                                     newTargetInfo->uri());
+
     if (targetInfo == toInfo) {
         completeSourceFiles.append(fromInfo->uri());
         precompleteTargetFileInfo.append(newTargetInfo);
